/*
 * Billboard CGI configuration file parser
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <time.h>
#include <sys/time.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include "library.h"
#include "vector.h"
#include "rope.h"
#include "cgi.h"
#include "smalloc.h"
#include "log.h"

#define TM_MIN	(0)
#define TM_HOUR	(1)
#define TM_MDAY	(2)
#define TM_MON	(3)
#define TM_WDAY	(4)

static struct tm *
config_tm(void)
{
	time_t		 timebuf;
	time(&timebuf);
	return gmtime(&timebuf);
}

int
config_checkparm(char *cp, struct tm *tm, int part)
{
	char	*dp;
	int		 start, end;
	int		 val;

	if (*cp == '*')
		return TRUE;

	/* Rely heavily on the definition of atol as initial part */
	start = atol(cp);

	/* Are we a <nn> or a <nn>-<mm> */
	dp = strchr(cp, '-');
	if (dp)
		end = atol(++dp);
	else
		end = start;

	switch(part) {
		case TM_MIN:	val = tm->tm_min;	break;
		case TM_HOUR:	val = tm->tm_hour;	break;
		case TM_MDAY:	val = tm->tm_mday;	break;
		case TM_MON:	val = tm->tm_mon;	break;
		case TM_WDAY:	val = tm->tm_wday;	break;
		default:		val = 0;	/* Defeat compiler warning */
	}

	if (start > val) return FALSE;
	if (end < val) return FALSE;
	return TRUE;	/* start <=  val && end >= val */
}

/* BEWARE: These three macros do NOT conform to semicolon macro rules */
#define NEXT_PARM() \
	cp = ep;					\
	while (isspace(*cp)) cp++;	\
	ep = strpbrk(cp, " \t\n");

#define CHECK_PARM() \
	if (!ep) {										\
		log("cgiconf", PRI_ERR, "Insufficient fields on line '%s'", \
						line); \
		continue;									\
	}

#define TEST_PARM(x) if (!config_checkparm(cp, tm, x)) continue;

#define DO_PARM(x)	\
	NEXT_PARM();	\
	CHECK_PARM();	\
	TEST_PARM(x);

vector_t *
config_names(rope_t *config, struct tm *tm)
{
	FILE		*input;
	char		 line[BUFSIZ];
	char		*cp, *ep;
	vector_t	*v;

	v = vector_new(sfree, 0);

	input = fopen(rope_chars(config), "r");
	if (!input) {
		log("cgiconf", PRI_CRIT, "Could not open config file '%s': %s",
						rope_chars(config), strerror(errno));
		return v;	/* Must not fail horribly */
	}

	while (fgets(line, BUFSIZ, input)) {
		/* Clobber */
		line[strcspn(line, "#\r\n")] = '\0';
		if (!strlen(line)) continue;

		ep = line;

		DO_PARM(TM_MIN);
		DO_PARM(TM_HOUR);
		DO_PARM(TM_MDAY);
		DO_PARM(TM_MON);
		DO_PARM(TM_WDAY);

		/* Step cp to our filename */
		NEXT_PARM();

		vector_push_back(v, sstrdup(cp));
	}

	return v;
}

vector_t *
config_vector(rope_t *config)
{
	struct tm	*tm;
	vector_t	*v;

	tm = config_tm();

	v = config_names(config, tm);

	return v;
}
