/*
 * C utility library
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "library.h"
#include "rope.h"
#include "smalloc.h"

rope_t *
rope_new(char *str)
{
	rope_t	*rope;
	rope = (rope_t *)smalloc(sizeof(rope_t));
	rope->chars = strdup(str);
	rope->size = strlen(str);
	rope->length = strlen(str);
	return rope;
}

void
rope_delete(rope_t *rope)
{
	free(rope->chars);
	free(rope);
}

char *
rope_chars(rope_t *rope)
{
	return rope->chars;
}

int
rope_length(rope_t *rope)
{
	return rope->length;
}

rope_t *
rope_add(rope_t *targ, char *src)
{
	int	 len;
	int	 newsize;

	newsize = ((targ->size > 0)?targ->size:32);	/* Safety */
	len = targ->length + strlen(src);
	while (len > newsize)
		newsize *= 2;
	
	if (newsize > targ->size) {
		targ->chars = (char *)srealloc(targ->chars, newsize + 1);
		targ->size = newsize;
	}
	strcpy(&(targ->chars[targ->length]), src);
	targ->length = len;
	
	return targ;
}

rope_t *
rope_concat(rope_t *targ, rope_t *src)
{
	return rope_add(targ, src->chars);
}

void
rope_set(rope_t *targ, char *str)
{
	targ->length = 0;
	rope_add(targ, str);
}

char *
rope_substr(rope_t *rope, int start, int end)
{
	int		 len;
	char	*str;
	

	if (start < 0)
		start = 0;
	if (end > rope->length)
		end = rope->length;
	if (end < start)
		end = start;
	
	len = end - start;
	str = (char *)smalloc(len + 1);
	strncpy(str, &(rope->chars[start]), len);
	str[len] = '\0';
	return str;
}

void
rope_replace(rope_t *rope, char *old, char *new)
{
}
