/*
 * Recorded redirection handler
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <errno.h>
#include "library.h"
#include "cgi.h"
#include "smalloc.h"
#include "hash.h"
#include "log.h"

#define DIR_RELOCATE	DIR_DATA "/"
#define LOGFILE			"/relocate/logfiles/relocate.log"

void
log_file(char *host, char *from, char *to)
{
	FILE	*output;
	time_t	 timebuf;
	char	*path;
	int		 len;

	len = strlen(DIR_RELOCATE) + strlen(host) + strlen(LOGFILE) + 1;
	path = (char *)smalloc(len);
	sprintf(path, DIR_RELOCATE "%s" LOGFILE, host);

	time(&timebuf);

	output = fopen(path, "a");
	if (!output) {
		log("cgisys", PRI_ERR, "Could not append to '%s': %s",
						path, strerror(errno));
		return;
	}

	fprintf(output, "%ld %s %s\n", timebuf, from, to);

	fclose(output);
}

void
invoke (void)
{
	char	*host;
	char	*from;
	char	*to;
	void	*cgihash;

	cgihash = cgi_hash_new();

	host = getenv("SERVER_NAME");
	if (!host) host = DEFAULT;

	to = hash_get(cgihash, "ToURL");
	if (!to) {
		log("cgiconf", PRI_CRIT, "No ToURL field in CGI args");
		exit(0);
	}

	printf("Location: %s\n\n", to);
	fclose(stdout);		/* Flush to browser */

	from = hash_get(cgihash, "FromURL");
	if (!from) {
		// log("cgiconf", PRI_ERR, "No 'FromURL' field in hash table");
		from = getenv("HTTP_REFERER");
		if (!from) {
			log("cgiconf", PRI_ERR, "No HTTP_REFERER (or FromURL)");
			exit(0);
		}
	}

	log_file(host, from, to);
}
