/*
 * Library test suite
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "library.h"
#include "hash.h"
#include "cgi.h"
#include "rope.h"
#include "list.h"
#include "utils.h"
#include "log.h"
#include "wregex.h"
#include "smalloc.h"

#define H_ADD(x, y) hash_add(hash, x, strdup(y))
#define H_GET(x) ((char *)(hash_get(hash, x)))
#define H_DEL(x) (hash_remove(hash, x))
#define STR "||b|bakph|Park Hyatt Baku|Park Hyatt Baku|Baku,Azerbaijan|Baku|International Park|Hotel|ok|||||Sumgait Caspian Sea|"

void
test_hash (void)
{
	hash_t	*hash;

	hash = hash_new(sfree, 1);
	H_ADD("foo", "bar");
	printf("foo = %s  (should be 'bar')\n", H_GET("foo"));
	H_ADD("abc", "abc");
	printf("abc = %s  (should be 'abc')\n", H_GET("abc"));
	printf("foo = %s  (should be 'bar')\n", H_GET("foo"));
	H_ADD("foo", "qux");
	printf("foo = %s  (should be 'qux')\n", H_GET("foo"));
	H_DEL("foo");
	printf("foo = %s  (should be '(null)')\n", H_GET("foo"));
}

void
test_explode(void)
{
	vector_t	*v;
	char		*s;
	char		*t;

	v = explode(STR, "|");
	s = implode(v, "|");
	t = implode(v, "|");

	printf("Line is %s\n", STR);
	printf("Str  is %s\n", s);
	printf("Str  is %s\n", t);
}

void
test_log(void)
{
	log("error404", 7, "Foo: %d %s", 7, "bar");
}

void
test_vector(void)
{
	vector_t	*v;
	v = vector_new(NULL, 0);
	vector_push_back(v, "0");
	printf("Push back;\tVector is %s\n", implode(v, ", "));
	vector_push_back(v, "1");
	printf("Push back;\tVector is %s\n", implode(v, ", "));
	vector_insert(v, "2", 1);
	printf("Insert 1;\tVector is %s\n", implode(v, ", "));
	vector_push_front(v, "3");
	printf("Push front;\tVector is %s\n", implode(v, ", "));
	vector_insert(v, "4", 2);
	printf("Insert 2;\tVector is %s\n", implode(v, ", "));
	vector_pop(v, 3);
	printf("Pop 3;   \tVector is %s\n", implode(v, ", "));
	vector_pop_front(v);
	printf("Pop front;\tVector is %s\n", implode(v, ", "));
	vector_pop_back(v);
	printf("Pop back;\tVector is %s\n", implode(v, ", "));
	vector_remove(v, 0);
	printf("Remove 0;\tVector is %s\n", implode(v, ", "));
}

void
test_wregex(void)
{
	wregex_t	*r;
	int			 i;
	r = wregex_new("f.o", 0);
	i = wregex_execute(r, "foo");
	printf("Execute of wregex returned %d\n", i);
	i = wregex_execute(r, "bar");
	printf("Execute of wregex returned %d\n", i);
	wregex_delete(r);
}


int
main (void)
{
	test_hash();
	test_explode();
	test_log();
	test_vector();
	test_wregex();

	return 0;
}
