%{
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "library.h"
#include "rope.h"
#include "hash.h"
#include "log.h"

/* Must be bits and must agree with main.c */
#define VAL_BLANK	(0)
#define VAL_USER	(1)
#define VAL_COMP	(2)

void template_print(int i);
void template_message(void);
int square_state(char *square);

char	*yysquare;
int		 val_user;
int		 val_comp;
int		 val_state;
char	*yytemplate;
hash_t	*yyhash;

/* Note, square_state is in main.c, this is not brilliant, but it
 * might as well be there as we have either a remote call to
 * square_state or a call to square_win. */

%}

%option nounput
%option noyywrap
%option caseless

%%

\{Square0\}	{	template_print(0);	}
\{Square1\}	{	template_print(1);	}
\{Square2\}	{	template_print(2);	}
\{Square3\}	{	template_print(3);	}
\{Square4\}	{	template_print(4);	}
\{Square5\}	{	template_print(5);	}
\{Square6\}	{	template_print(6);	}
\{Square7\}	{	template_print(7);	}
\{Square8\}	{	template_print(8);	}
\{Message\}	{	template_message();	}

%%

void
template_image(int i)
{
	fprintf(yyout, "<IMG SRC=\"" DIR_RESOURCE
					"/tictactoe/%s/symbols/%d.gif\" BORDER=0 "
					"HSPACE=0 VSPACE=0>",
					yytemplate, i);
}

void
template_print(int i)
{
	char	*name;

	name = getenv(ENV_SCRIPT);
	if (!name) {
		log("cgisys", PRI_WARNING, "Script could not getenv(\""
						ENV_SCRIPT "\") for base of URL - guessing");
		name = "/cgi-bin/tictactoe.cgi";
	}

	if (yysquare[i] == 0) {
		if (val_state == 0) {
			fprintf(yyout, "<A HREF=\"%s?", name);
			fprintf(yyout, "u=%d", val_user | (1 << i));
			fprintf(yyout, "&c=%d&case=%s\">", val_comp, yytemplate);
			template_image(0);
			fprintf(yyout, "</A>");
		}
		else {
			template_image(0);
		}
	}
	else {
		template_image(yysquare[i]);
	}
}

void
template_message(void)
{
	char	*message;

	/* Hook for future */
	message = hash_get(yyhash, "Message");
	if (message)
		fprintf(yyout, "Message: %s<BR>\n", message);

	switch(val_state) {
		case VAL_USER:
			fprintf(yyout, "User wins<BR>\n");
			break;
		case VAL_COMP:
			fprintf(yyout, "Computer wins<BR>\n");
			break;
		case -1:
			fprintf(yyout, "Game is a draw<BR>\n");
			break;
		case 0:
			fprintf(yyout, "Play on!<BR>\n");
			break;
		default:
			fprintf(yyout, "Unknown game state %d<BR>\n", val_state);
			log("cgisys", PRI_WARNING, "Unknown game state %d",
							val_state);
	}
}

FILE *
template_stream(char *template)
{
	rope_t	*rope;
	char	*hostname;
	FILE	*input;

	hostname = getenv(ENV_HOSTNAME);
	if (!hostname) {
		log("cgiconf", PRI_CRIT, "No " ENV_HOSTNAME " in environment");
		exit(0);
	}

	rope = rope_new(DIR_DATA "/");
	rope_add(rope, hostname);
	rope_add(rope, "/tictactoe/templates/");
	rope_add(rope, template);
	rope_add(rope, ".tpt");

	input = fopen(rope_chars(rope), "r");
	if (!input) {
		log("cgiconf", PRI_CRIT, "Could not open template '%s': %s",
						rope_chars(rope), strerror(errno));
		exit(0);
	}

	// rope_delete(rope);	/* Not worth it */

	return input;
}

void
template_parse(hash_t *cgihash, char *square, int vu, int vc)
{
	char	*template;

	template = hash_get(cgihash, "case");
	if (!template) {
		log("cgiconf", PRI_CRIT, "No 'case' in hash table");
		exit(0);
	}
	template[strcspn(template, "/")] = 0;

	yyhash = cgihash;
	yysquare = square;
	val_user = vu;
	val_comp = vc;
	val_state = square_state(square);
	yytemplate = template;
	yyin = template_stream(template);

	yyout = stdout;
	fprintf(yyout, "Content-type: text/html\n\n");
	yylex();
}
